package org.average.whereami;

import org.average.whereami.PersistentStore;
import org.average.whereami.SayWhen;

import java.lang.Math;
import java.io.IOException;

import com.google.api.client.googleapis.json.GoogleJsonResponseException;
import com.google.api.client.http.HttpResponseException;
import com.google.api.services.latitude.Latitude;
import com.google.api.services.latitude.LatitudeRequest;
import com.google.api.services.latitude.model.Location;

import android.content.Context;
import android.content.res.Resources;
import android.text.format.Time;
import android.util.Log;

public final class LastLocation extends Oracle {

	final String TAG = getClass().getName();

	private String[] loc_names;
	private String[] loc_lats;
	private String[] loc_lons;
	private String[] loc_dists;
	private String myname;
	private String registered;
	private SayWhen sayWhen;

	public LastLocation(final Resources res, final PersistentStore store) {
		super(res, store);
		loc_names = res.getStringArray(R.array.loc_names);
		loc_lats = res.getStringArray(R.array.loc_lats);
		loc_lons = res.getStringArray(R.array.loc_lons);
		loc_dists = res.getStringArray(R.array.loc_dists);
		myname = res.getString(R.string.myname);
		registered = res.getString(R.string.registered);
		sayWhen = new SayWhen(res);
	}

	@Override
	public final String getResult() {
		Latitude latitude = Latitude.builder(httpTransport, jsonFactory)
			.setHttpRequestInitializer(accessProtectedResource())
			.setApplicationName("WhereAmI/1.0").build();
		try {
			Log.v(TAG, "entering getResult");
			Latitude.CurrentLocation.Get request =
				latitude.currentLocation().get();
			request.setGranularity("best");
			Location cloc = request.execute();
			Double lat = Double.parseDouble(
					cloc.getLatitude().toString());
			Double lon = Double.parseDouble(
					cloc.getLongitude().toString());
			Long tsm = Long.parseLong(
					cloc.getTimestampMs().toString());
			return locationMessage(lat, lon, tsm);
		} catch (GoogleJsonResponseException e) {
			Log.e(TAG, "GoogleJsonResponseException: " + e);
			if (e.getResponse().getStatusCode() == 401) {
				return authErrorMessage;
			} else {
				return e.getMessage();
			}
                } catch (HttpResponseException e) {
			Log.e(TAG, "HttpResponseException: " + e);
			return e.getMessage();
		} catch (IOException e) {
			e.printStackTrace();
			return e.getMessage();
                }
	}

	private String locationMessage(Double lat, Double lon, Long tsm) {
		Log.v(TAG, "entering locationMessage(" + lat +
				"," + lon + "," + tsm + ")");
		String locmsg = "<UNSET>";
		String agomsg = "<UNSET>";
		for (int i = 0; i < loc_names.length; i++) {
			Double dist = haversine(lat, lon,
					Double.parseDouble(loc_lats[i]),
					Double.parseDouble(loc_lons[i]));
			Log.v(TAG, "Dist from " + loc_names[i] +
				" is " + dist);
			if ((loc_dists[i] == "0") ||
			    (dist < Double.parseDouble(loc_dists[i]))) {
				locmsg = loc_names[i];
				break;
			}
		}
		return myname + " " + locmsg + ", " + registered +
			" " + sayWhen.say(tsm);
	}

	private Double toRad(Double deg) {
		return deg * 3.14159265 / 180;
	}

	private Double haversine(Double lat1, Double lon1,
				Double lat2, Double lon2) {
		Double dlat = toRad(lat2 - lat1);
		Double dlon = toRad(lon2 - lon1);
		Double a = Math.sin(dlat / 2) * Math.sin(dlat / 2) +
			Math.sin(dlon / 2) * Math.sin(dlon / 2) *
			Math.cos(toRad(lat1)) * Math.cos(toRad(lat2));
		Double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
		return 6371 * c;
	}
}
